﻿/*********************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2010 Manish Sinha <mail@manishsinha.net>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:

 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 ********************************************************************/


namespace LpNet.WadlSharp.Common.Generator
{
    using System.CodeDom;
    using System.Collections.Generic;
    using Enum;
    using Interface.Generator;
    using Resources;

    public class LpClass : IClass
    {
        #region IClass Members

        /// <summary>
        /// Create a CodeDeclarationType (Class)
        /// </summary>
        /// <param name="repType">The representation instance which has non-null params Property</param>
        /// <param name="nameSpaces">The list of namespaces import required in the final generated code</param>
        /// <param name="ns">The root namespace for all the generated Types</param>
        /// <returns>Instance of CodeTypeDeclaration</returns>
        public CodeTypeDeclaration Create(representation_type repType, HashSet<string> nameSpaces, CodeNamespace ns)
        {
            CodeTypeDeclaration cdDec= new CodeTypeDeclaration();

            cdDec.Name = WadlSharpUtils.CreateIdentifier(repType.id);
            cdDec.IsClass = true;
            cdDec.Attributes = MemberAttributes.Public | MemberAttributes.Static;
            cdDec.IsPartial = true;

            #region Creating the XML Comments for this class

            IComment comment = new LpComment();
            if (repType.doc != null)
                foreach (doc doc in repType.doc)
                {
                    cdDec.Comments.AddRange(comment.Create(doc, LanguageConstructType.Class,repType.id));
                }

            #endregion

            #region Creating the Properties

            if (repType.param != null)
            {
                List<CodeMemberProperty> propertiesList= new List<CodeMemberProperty>();
                List<CodeMemberField> fieldsList = new List<CodeMemberField>();

                // Iterate over all the Properties
                foreach (param param in repType.param)
                {
                    CodeMemberField field;

                    IProperty property = new LpProperty();
                    CodeMemberProperty prop = property.Create(param, out field, nameSpaces, ns);

                    propertiesList.Add(prop);
                    fieldsList.Add(field);
                }

                // Add the Fields and Properties for this Type
                if (propertiesList.Count > 0)
                {
                    //cdDec.StartDirectives.Add(new CodeRegionDirective(CodeRegionMode.Start, "Public Properties"));
                    cdDec.Members.AddRange(propertiesList.ToArray());
                    //cdDec.EndDirectives.Add(new CodeRegionDirective(CodeRegionMode.End, string.Empty));
                }

                if (fieldsList.Count > 0)
                {
                    //cdDec.StartDirectives.Add(new CodeRegionDirective(CodeRegionMode.Start, "Private Fields"));
                    cdDec.Members.AddRange(fieldsList.ToArray());
                    //cdDec.EndDirectives.Add(new CodeRegionDirective(CodeRegionMode.End, string.Empty));
                }

            }

            #endregion

            return cdDec;
        }

        public CodeTypeDeclaration Create(resource_type resourceType, HashSet<string> nameSpaces, CodeNamespace ns)
        {
            CodeTypeDeclaration typeDeclaration = new CodeTypeDeclaration();

            #region The Class Definition and Properties

            typeDeclaration.Name = WadlSharpUtils.CreateIdentifier(resourceType.id);
            typeDeclaration.IsClass = true;
            typeDeclaration.Attributes = MemberAttributes.Public | MemberAttributes.Static;
            typeDeclaration.IsPartial = true;

            #endregion

            #region Creating the XML Comments for the class description

            IComment comment = new LpComment();
            if (resourceType.doc != null)
                foreach (doc doc in resourceType.doc)
                {
                    typeDeclaration.Comments.AddRange(
                                                        comment.Create(
                                                                        doc, 
                                                                        LanguageConstructType.Class, 
                                                                        resourceType.id
                                                                     )
                                                      );
                }

            #endregion

            #region Methods for this Class

            if (resourceType.method != null && resourceType.method.Length > 0)
            {
                //typeDeclaration.StartDirectives.Add(new CodeRegionDirective(CodeRegionMode.Start, "Public Methods"));

                foreach (method method in resourceType.method)
                {
                    IMethod lpmethod = new LpMethod();
                    CodeMemberMethod memberMethod;

                    if (method.request != null && method.request.representation != null)
                    {
                        foreach (representation_type representation in method.request.representation)
                        {
                            memberMethod = lpmethod.Create(method, representation, nameSpaces, ns);
                            typeDeclaration.Members.Add(memberMethod);
                        }
                    }
                    else
                    {
                        memberMethod = lpmethod.Create(method, nameSpaces, ns);
                        typeDeclaration.Members.Add(memberMethod);
                    }
                }

                //typeDeclaration.EndDirectives.Add(new CodeRegionDirective(CodeRegionMode.End, string.Empty));
            }

            #endregion

            #region Properties for this Class

            if(resourceType.param != null)
            {
                foreach (param param in resourceType.param)
                {
                    IProperty paramInst = new LpProperty();
                    CodeMemberField field;

                    CodeMemberProperty prop = paramInst.Create(param, out field, nameSpaces, ns);

                    typeDeclaration.Members.Add(prop);
                    typeDeclaration.Members.Add(field);
                }
            }

            #endregion

            #region Passing the list of top level resources to respective resource_type

            bool isResourceRoot = false;

            // Check for each resource in the list of resources
            foreach (resource resource in ResourceList.Resource.resource)
            {
                // resource.path is optional. For more details check http://www.w3.org/Submission/wadl/#x3-110002.5
                if (resource.type != null)
                {
                    foreach (string s in resource.type)
                    {
                        string href = WadlSharpUtils.CreateAnchor(s);
                        // Create the resource_type.id
                        href = WadlSharpUtils.CreateIdentifier(href);

                        if (string.Equals(href, typeDeclaration.Name))
                        {
                            /*CodeTypeConstructor constructor = new CodeTypeConstructor();
                            constructor.Attributes = MemberAttributes.Public | MemberAttributes.Final;*/

                            IField uriField = new LpField();
                            CodeMemberField field = uriField.Create("_uri", typeof (string), nameSpaces);
                            field.InitExpression = new CodeFieldReferenceExpression(null,
                                                                                    "\"" + ResourceList.Resource.@base +
                                                                                    "\"");
                            typeDeclaration.Members.Add(field);

                            isResourceRoot = true;

                            /*constructor.Statements.Add(
                                                        new CodeAssignStatement(
                                                                                new CodeFieldReferenceExpression(
                                                                                                                 new CodeThisReferenceExpression(),
                                                                                                                 "_baseUri"
                                                                                                               ),
                                                                                new CodeVariableReferenceExpression("\""+ResourceList.Resource.@base+"\"")
                                                                                )
                                                        );*/

                            //typeDeclaration.Members.Add(constructor);
                        }
                    }
                }
            }

            #endregion


            #region Creating the Constructor
            if (isResourceRoot != true)
            {
                IConstructor cons = new LpConstructor();
                CodeConstructor typeConst = cons.Create(true, true, true, typeDeclaration, nameSpaces, ns);
                typeDeclaration.Members.Add(typeConst);
            }
            else
            {
                IConstructor cons = new LpConstructor();
                CodeConstructor typeConst = cons.Create(false, true, true, typeDeclaration, nameSpaces, ns);
                typeDeclaration.Members.Add(typeConst);
            }

            #endregion

            return typeDeclaration;
        }

        #endregion
    }
}
