﻿/*********************************************************************
 * The MIT License (MIT)
 * 
 * Copyright (c) 2010 Manish Sinha <mail@manishsinha.net>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:

 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 ********************************************************************/


namespace LpNet.WadlSharp.Common.Generator
{
    using System;
    using System.CodeDom;
    using System.Collections.Generic;
    using System.Linq;
    using System.Xml;
    using Enum;
    using Interface.Generator;
    using Resources;

    public class LpComment : IComment
    {
        #region IComment Members

        public CodeCommentStatementCollection Create(doc commentInstance, 
                LanguageConstructType constructType, string constructName)
        {
            if (commentInstance == null)
                throw new ArgumentNullException("commentInstance");

            commentInstanceField = commentInstance;
            constructTypeField = constructType;

            commentCollField = new CodeCommentStatementCollection();

            // If the doc instance has some comments
            if (commentInstance.Any != null)
            {
                switch (constructType)
                {
                    #region Method and Property
                    case LanguageConstructType.Property:
                    case LanguageConstructType.Method:
                    case LanguageConstructType.Class:
                        {
                            CodeCommentStatement summary = new CodeCommentStatement("<summary>", true);
                            commentCollField.Add(summary);

                            foreach (XmlNode node in commentInstance.Any)
                            {
                                string filteredText = WadlSharpUtils.FilterComments(node.InnerText);
                                CodeCommentStatement commentStatement = new CodeCommentStatement(filteredText, true);
                                commentCollField.Add(commentStatement);
                            }

                            CodeCommentStatement summaryEnd = new CodeCommentStatement("</summary>", true);
                            commentCollField.Add(summaryEnd);
                            break;
                        }
                    #endregion

                    #region Input Parameters
                    case LanguageConstructType.InputParameter:
                        {
                            List<string> listOfComments = new List<string>();

                            foreach (XmlNode node in commentInstance.Any)
                            {
                                string filteredText = WadlSharpUtils.FilterComments(node.InnerText);
                                listOfComments.Add(filteredText);
                            }

                            string comments = string.Join(@"\n/// ", listOfComments.ToArray());
                            string inputParamComm = string.Format("<param name=\"{0}\">{1}</param>", constructName,comments);

                            CodeCommentStatement summaryEnd = new CodeCommentStatement(inputParamComm, true);
                            commentCollField.Add(summaryEnd);
                            break;
                        }
                    #endregion

                    #region Return Parameters
                    case LanguageConstructType.ReturnParameter:
                        {
                            List<string> listOfComments = new List<string>();

                            foreach (XmlNode node in commentInstance.Any)
                            {
                                string filteredText = WadlSharpUtils.FilterComments(node.InnerText);
                                listOfComments.Add(filteredText);
                            }

                            string comments = string.Join(@"\n/// ", listOfComments.ToArray());
                            string inputParamComm = string.Format("<returns>{0}</returns>", comments);

                            CodeCommentStatement summaryEnd = new CodeCommentStatement(inputParamComm, true);
                            commentCollField.Add(summaryEnd);
                            break;
                        }
                    #endregion
                }
            }

            return commentCollField;
        }

        /// <summary>
        /// The type of language construct used.
        /// Like Method, Property, Input Params, Output Params etc
        /// </summary>
        public LanguageConstructType ConstructType
        {
            get
            {
                return constructTypeField;
            }
        }

        /// <summary>
        /// The name of the comment instance
        /// </summary>
        public string Title
        {
            get
            {
                return commentInstanceField.title;
            }
        }

        public string Language
        {
            get
            {
                return commentInstanceField.lang;
            }
        }

        public List<XmlNode> Nodes
        {
            get
            {
                return commentInstanceField.Any.ToList();
            }
        }

        #endregion

        #region Private Fields

        private CodeCommentStatementCollection commentCollField;

        private doc commentInstanceField;

        private LanguageConstructType constructTypeField;

        #endregion
    }
}
